package com.sun.java.util.jar.pack;

import java.io.IOException;
import java.util.Arrays;

class Instruction
{
    protected byte[] bytes;
    protected int pc;
    protected int bc;
    protected int w;
    protected int length;
    protected boolean special;
    private static final byte[][] BC_LENGTH;
    private static final byte[][] BC_INDEX;
    private static final byte[][] BC_TAG;
    private static final byte[][] BC_BRANCH;
    private static final byte[][] BC_SLOT;
    private static final byte[][] BC_CON;
    private static final String[] BC_NAME;
    private static final String[][] BC_FORMAT;
    private static int BW;
    
    protected Instruction(byte[] array, int n, int n2, int n3, int n4) {
        this.reset(array, n, n2, n3, n4);
    }
    
    private void reset(byte[] bytes, int pc, int bc, int w, int length) {
        this.bytes = bytes;
        this.pc = pc;
        this.bc = bc;
        this.w = w;
        this.length = length;
    }
    
    public int getBC() {
        return this.bc;
    }
    
    public boolean isWide() {
        return this.w != 0;
    }
    
    public byte[] getBytes() {
        return this.bytes;
    }
    
    public int getPC() {
        return this.pc;
    }
    
    public int getLength() {
        return this.length;
    }
    
    public int getNextPC() {
        return this.pc + this.length;
    }
    
    public Instruction next() {
        int n = this.pc + this.length;
        if (n == this.bytes.length) {
            return null;
        }
        return at(this.bytes, n, this);
    }
    
    public boolean isNonstandard() {
        return isNonstandard(this.bc);
    }
    
    public void setNonstandardLength(int length) {
        assert !(!this.isNonstandard());
        this.length = length;
        return;
    }
    
    public Instruction forceNextPC(int n) {
        return new Instruction(this.bytes, this.pc, -1, -1, n - this.pc);
    }
    
    public static Instruction at(byte[] array, int n) {
        return at(array, n, null);
    }
    
    public static Instruction at(byte[] array, int n, Instruction instruction) {
        int n2 = getByte(array, n);
        int n3 = 0;
        int n4 = Instruction.BC_LENGTH[n3][n2];
        if (n4 == 0) {
            switch (n2) {
                case 196: {
                    n2 = getByte(array, n + 1);
                    n3 = 1;
                    n4 = Instruction.BC_LENGTH[n3][n2];
                    if (n4 == 0) {
                        n4 = 1;
                        break;
                    }
                    break;
                }
                case 170: {
                    return new Instruction.TableSwitch(array, n);
                }
                case 171: {
                    return new Instruction.LookupSwitch(array, n);
                }
                default: {
                    n4 = 1;
                    break;
                }
            }
        }
        assert n4 > 0;
        assert n + n4 <= array.length;
        if (instruction != null && !instruction.special) {
            instruction.reset(array, n, n2, n3, n4);
            return instruction;
        }
        return new Instruction(array, n, n2, n3, n4);
    }
    
    public byte getCPTag() {
        return Instruction.BC_TAG[this.w][this.bc];
    }
    
    public int getCPIndex() {
        int n = Instruction.BC_INDEX[this.w][this.bc];
        if (n == 0) {
            return -1;
        }
        assert this.w == 0;
        if (this.length == 2) {
            return getByte(this.bytes, this.pc + n);
        }
        return getShort(this.bytes, this.pc + n);
    }
    
    public void setCPIndex(int n) {
        int n2 = Instruction.BC_INDEX[this.w][this.bc];
        assert n2 != 0;
        if (this.length == 2) {
            setByte(this.bytes, this.pc + n2, n);
        }
        else {
            setShort(this.bytes, this.pc + n2, n);
        }
        assert this.getCPIndex() == n;
        return;
    }
    
    public ConstantPool.Entry getCPRef(ConstantPool.Entry[] array) {
        int cpIndex = this.getCPIndex();
        return (cpIndex < 0) ? null : array[cpIndex];
    }
    
    public int getLocalSlot() {
        int n = Instruction.BC_SLOT[this.w][this.bc];
        if (n == 0) {
            return -1;
        }
        if (this.w == 0) {
            return getByte(this.bytes, this.pc + n);
        }
        return getShort(this.bytes, this.pc + n);
    }
    
    public int getBranchLabel() {
        int n = Instruction.BC_BRANCH[this.w][this.bc];
        if (n == 0) {
            return -1;
        }
        assert this.w == 0;
        assert this.length == 5;
        int int1;
        if (this.length == 3) {
            int1 = (short)getShort(this.bytes, this.pc + n);
        }
        else {
            int1 = getInt(this.bytes, this.pc + n);
        }
        assert int1 + this.pc >= 0;
        assert int1 + this.pc <= this.bytes.length;
        return int1 + this.pc;
    }
    
    public void setBranchLabel(int n) {
        int n2 = Instruction.BC_BRANCH[this.w][this.bc];
        assert n2 != 0;
        if (this.length == 3) {
            setShort(this.bytes, this.pc + n2, n - this.pc);
        }
        else {
            setInt(this.bytes, this.pc + n2, n - this.pc);
        }
        assert n == this.getBranchLabel();
        return;
    }
    
    public int getConstant() {
        int n = Instruction.BC_CON[this.w][this.bc];
        if (n == 0) {
            return 0;
        }
        switch (this.length - n) {
            case 1: {
                return (byte)getByte(this.bytes, this.pc + n);
            }
            case 2: {
                return (short)getShort(this.bytes, this.pc + n);
            }
            default: {
                assert false;
                return 0;
            }
        }
    }
    
    public void setConstant(int n) {
        int n2 = Instruction.BC_CON[this.w][this.bc];
        assert n2 != 0;
        switch (this.length - n2) {
            case 1: {
                setByte(this.bytes, this.pc + n2, n);
                break;
            }
            case 2: {
                setShort(this.bytes, this.pc + n2, n);
                break;
            }
        }
        assert n == this.getConstant();
        return;
    }
    
    @Override
    public boolean equals(Object o) {
        return o != null && o.getClass() == Instruction.class && this.equals((Instruction)o);
    }
    
    @Override
    public int hashCode() {
        return 11 * (11 * (11 * (11 * (11 * 3 + Arrays.hashCode(this.bytes)) + this.pc) + this.bc) + this.w) + this.length;
    }
    
    public boolean equals(Instruction instruction) {
        if (this.pc != instruction.pc) {
            return false;
        }
        if (this.bc != instruction.bc) {
            return false;
        }
        if (this.w != instruction.w) {
            return false;
        }
        if (this.length != instruction.length) {
            return false;
        }
        int i = 1;
        while (i < this.length) {
            if (this.bytes[this.pc + i] != instruction.bytes[instruction.pc + i]) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    static String labstr(int i) {
        if (i >= 0 && i < 100000) {
            return (100000 + i + "").substring(1);
        }
        return i + "";
    }
    
    @Override
    public String toString() {
        return this.toString((ConstantPool$Entry[])null);
    }
    
    public String toString(ConstantPool.Entry[] array) {
        String s = labstr(this.pc) + ": ";
        if (this.bc >= 202) {
            return s + Integer.toHexString(this.bc);
        }
        if (this.w == 1) {
            s += "wide ";
        }
        String str = (this.bc < Instruction.BC_NAME.length) ? Instruction.BC_NAME[this.bc] : null;
        if (str == null) {
            return s + "opcode#" + this.bc;
        }
        String str2 = s + str;
        int cpTag = this.getCPTag();
        if (cpTag != 0) {
            str2 = str2 + " " + ConstantPool.tagName(cpTag) + ":";
        }
        int cpIndex = this.getCPIndex();
        if (cpIndex >= 0) {
            str2 += ((array == null) ? ("" + cpIndex) : ("=" + array[cpIndex].stringValue()));
        }
        int localSlot = this.getLocalSlot();
        if (localSlot >= 0) {
            str2 = str2 + " Local:" + localSlot;
        }
        int branchLabel = this.getBranchLabel();
        if (branchLabel >= 0) {
            str2 = str2 + " To:" + labstr(branchLabel);
        }
        int constant = this.getConstant();
        if (constant != 0) {
            str2 = str2 + " Con:" + constant;
            return str2;
        }
        return str2;
    }
    
    public int getIntAt(int n) {
        return getInt(this.bytes, this.pc + n);
    }
    
    public int getShortAt(int n) {
        return getShort(this.bytes, this.pc + n);
    }
    
    public int getByteAt(int n) {
        return getByte(this.bytes, this.pc + n);
    }
    
    public static int getInt(byte[] array, int n) {
        return (getShort(array, n + 0) << 16) + (getShort(array, n + 2) << 0);
    }
    
    public static int getShort(byte[] array, int n) {
        return (getByte(array, n + 0) << 8) + (getByte(array, n + 1) << 0);
    }
    
    public static int getByte(byte[] array, int n) {
        return array[n] & 0xFF;
    }
    
    public static void setInt(byte[] array, int n, int n2) {
        setShort(array, n + 0, n2 >> 16);
        setShort(array, n + 2, n2 >> 0);
    }
    
    public static void setShort(byte[] array, int n, int n2) {
        setByte(array, n + 0, n2 >> 8);
        setByte(array, n + 1, n2 >> 0);
    }
    
    public static void setByte(byte[] array, int n, int n2) {
        array[n] = (byte)n2;
    }
    
    public static boolean isNonstandard(int n) {
        return Instruction.BC_LENGTH[0][n] < 0;
    }
    
    public static int opLength(int n) {
        int n2 = Instruction.BC_LENGTH[0][n];
        assert n2 > 0;
        return n2;
    }
    
    public static int opWideLength(int n) {
        int n2 = Instruction.BC_LENGTH[1][n];
        assert n2 > 0;
        return n2;
    }
    
    public static boolean isLocalSlotOp(int n) {
        return n < Instruction.BC_SLOT[0].length && Instruction.BC_SLOT[0][n] > 0;
    }
    
    public static boolean isBranchOp(int n) {
        return n < Instruction.BC_BRANCH[0].length && Instruction.BC_BRANCH[0][n] > 0;
    }
    
    public static boolean isCPRefOp(int n) {
        return (n < Instruction.BC_INDEX[0].length && Instruction.BC_INDEX[0][n] > 0) || (n >= 233 && n < 242) || (n == 242 || n == 243);
    }
    
    public static byte getCPRefOpTag(int n) {
        if (n < Instruction.BC_INDEX[0].length && Instruction.BC_INDEX[0][n] > 0) {
            return Instruction.BC_TAG[0][n];
        }
        if (n >= 233 && n < 242) {
            return 51;
        }
        if (n == 243 || n == 242) {
            return 11;
        }
        return 0;
    }
    
    public static boolean isFieldOp(int n) {
        return n >= 178 && n <= 181;
    }
    
    public static boolean isInvokeInitOp(int n) {
        return n >= 230 && n < 233;
    }
    
    public static boolean isSelfLinkerOp(int n) {
        return n >= 202 && n < 230;
    }
    
    public static String byteName(int i) {
        String s;
        if (i < Instruction.BC_NAME.length && Instruction.BC_NAME[i] != null) {
            s = Instruction.BC_NAME[i];
        }
        else if (isSelfLinkerOp(i)) {
            int n2 = i - 202;
            int n3 = n2 >= 14;
            if (!(n3 == 0)) {
                n2 -= 14;
            }
            int n4 = n2 >= 7;
            if (!(n4 == 0)) {
                n2 -= 7;
            }
            int n5 = 178 + n2;
            assert n5 >= 178 && n5 <= 184;
            String s2 = Instruction.BC_NAME[n5] + (n3 ? "_super" : "_this");
            if (!(n4 == 0)) {
                s2 = "aload_0&" + s2;
            }
            s = "*" + s2;
        }
        else if (isInvokeInitOp(i)) {
            int n = i - 230;
            switch (n) {
                case 0: {
                    s = "*invokespecial_init_this";
                    break;
                }
                case 1: {
                    s = "*invokespecial_init_super";
                    break;
                }
                default: {
                    assert n == 2;
                    s = "*invokespecial_init_new";
                    break;
                }
            }
        }
        else {
            switch (i) {
                case 234: {
                    s = "*ildc";
                    break;
                }
                case 235: {
                    s = "*fldc";
                    break;
                }
                case 237: {
                    s = "*ildc_w";
                    break;
                }
                case 238: {
                    s = "*fldc_w";
                    break;
                }
                case 239: {
                    s = "*dldc2_w";
                    break;
                }
                case 233: {
                    s = "*cldc";
                    break;
                }
                case 236: {
                    s = "*cldc_w";
                    break;
                }
                case 240: {
                    s = "*qldc";
                    break;
                }
                case 241: {
                    s = "*qldc_w";
                    break;
                }
                case 254: {
                    s = "*byte_escape";
                    break;
                }
                case 253: {
                    s = "*ref_escape";
                    break;
                }
                case 255: {
                    s = "*end";
                    break;
                }
                default: {
                    s = "*bc#" + i;
                    break;
                }
            }
        }
        return s;
    }
    
    private static void def(String s, int n) {
        def(s, n, n);
    }
    
    private static void def(String s, int i, int n) {
        String[] array = { s, null };
        if (s.indexOf(119) > 0) {
            array[1] = s.substring(s.indexOf(119));
            array[0] = s.substring(0, s.indexOf(119));
        }
        for (int j = 0; j <= 1; ++j) {
            s = array[j];
            if (s != null) {
                int length = s.length();
                int max = Math.max(0, s.indexOf(107));
                int n2 = 0;
                int max2 = Math.max(0, s.indexOf(111));
                int max3 = Math.max(0, s.indexOf(108));
                int max4 = Math.max(0, s.indexOf(120));
                if (max > 0 && max + 1 < length) {
                    switch (s.charAt(max + 1)) {
                        case 'c': {
                            n2 = 7;
                            break;
                        }
                        case 'k': {
                            n2 = 51;
                            break;
                        }
                        case 'f': {
                            n2 = 9;
                            break;
                        }
                        case 'm': {
                            n2 = 10;
                            break;
                        }
                        case 'i': {
                            n2 = 11;
                            break;
                        }
                        case 'y': {
                            n2 = 18;
                            break;
                        }
                    }
                    assert n2 != 0;
                }
                else if (max > 0 && length == 2) {
                    assert i == 18;
                    n2 = 51;
                }
                while (i <= n) {
                    Instruction.BC_FORMAT[j][i] = s;
                    assert Instruction.BC_LENGTH[j][i] == -1;
                    Instruction.BC_LENGTH[j][i] = (byte)length;
                    Instruction.BC_INDEX[j][i] = (byte)max;
                    Instruction.BC_TAG[j][i] = (byte)n2;
                    assert n2 == 0;
                    Instruction.BC_BRANCH[j][i] = (byte)max2;
                    Instruction.BC_SLOT[j][i] = (byte)max3;
                    assert max3 == 0;
                    assert max == 0;
                    assert max == 0;
                    Instruction.BC_CON[j][i] = (byte)max4;
                    ++i;
                }
            }
        }
    }
    
    public static void opcodeChecker(byte[] array, ConstantPool.Entry[] array2, Package.Version version) throws Instruction.FormatException {
        Instruction obj = at(array, 0);
        while (obj != null) {
            int bc = obj.getBC();
            if (bc < 0 || bc > 201) {
                throw new Instruction.FormatException("illegal opcode: " + bc + " " + obj);
            }
            ConstantPool.Entry cpRef = obj.getCPRef(array2);
            if (cpRef != null) {
                int cpTag = obj.getCPTag();
                int tagMatches = cpRef.tagMatches(cpTag);
                if (tagMatches == 0 && (obj.bc == 183 || obj.bc == 184) && cpRef.tagMatches(11) && version.greaterThan(Constants.JAVA7_MAX_CLASS_VERSION)) {
                    tagMatches = 1;
                }
                if (tagMatches == 0) {
                    throw new Instruction.FormatException("illegal reference, expected type=" + ConstantPool.tagName(cpTag) + ": " + obj.toString(array2));
                }
            }
            obj = obj.next();
        }
    }
    
    static {
        BC_LENGTH = new byte[2][256];
        BC_INDEX = new byte[2][256];
        BC_TAG = new byte[2][256];
        BC_BRANCH = new byte[2][256];
        BC_SLOT = new byte[2][256];
        BC_CON = new byte[2][256];
        BC_NAME = new String[256];
        BC_FORMAT = new String[2][202];
        for (int i = 0; i < 202; ++i) {
            Instruction.BC_LENGTH[0][i] = -1;
            Instruction.BC_LENGTH[1][i] = -1;
        }
        def("b", 0, 15);
        def("bx", 16);
        def("bxx", 17);
        def("bk", 18);
        def("bkk", 19, 20);
        def("blwbll", 21, 25);
        def("b", 26, 53);
        def("blwbll", 54, 58);
        def("b", 59, 131);
        def("blxwbllxx", 132);
        def("b", 133, 152);
        def("boo", 153, 168);
        def("blwbll", 169);
        def("", 170, 171);
        def("b", 172, 177);
        def("bkf", 178, 181);
        def("bkm", 182, 184);
        def("bkixx", 185);
        def("bkyxx", 186);
        def("bkc", 187);
        def("bx", 188);
        def("bkc", 189);
        def("b", 190, 191);
        def("bkc", 192, 193);
        def("b", 194, 195);
        def("", 196);
        def("bkcx", 197);
        def("boo", 198, 199);
        def("boooo", 200, 201);
        for (int j = 0; j < 202; ++j) {
            if (Instruction.BC_LENGTH[0][j] != -1 && Instruction.BC_LENGTH[1][j] == -1) {
                Instruction.BC_LENGTH[1][j] = (byte)(1 + Instruction.BC_LENGTH[0][j]);
            }
        }
        String substring = "nop aconst_null iconst_m1 iconst_0 iconst_1 iconst_2 iconst_3 iconst_4 iconst_5 lconst_0 lconst_1 fconst_0 fconst_1 fconst_2 dconst_0 dconst_1 bipush sipush ldc ldc_w ldc2_w iload lload fload dload aload iload_0 iload_1 iload_2 iload_3 lload_0 lload_1 lload_2 lload_3 fload_0 fload_1 fload_2 fload_3 dload_0 dload_1 dload_2 dload_3 aload_0 aload_1 aload_2 aload_3 iaload laload faload daload aaload baload caload saload istore lstore fstore dstore astore istore_0 istore_1 istore_2 istore_3 lstore_0 lstore_1 lstore_2 lstore_3 fstore_0 fstore_1 fstore_2 fstore_3 dstore_0 dstore_1 dstore_2 dstore_3 astore_0 astore_1 astore_2 astore_3 iastore lastore fastore dastore aastore bastore castore sastore pop pop2 dup dup_x1 dup_x2 dup2 dup2_x1 dup2_x2 swap iadd ladd fadd dadd isub lsub fsub dsub imul lmul fmul dmul idiv ldiv fdiv ddiv irem lrem frem drem ineg lneg fneg dneg ishl lshl ishr lshr iushr lushr iand land ior lor ixor lxor iinc i2l i2f i2d l2i l2f l2d f2i f2l f2d d2i d2l d2f i2b i2c i2s lcmp fcmpl fcmpg dcmpl dcmpg ifeq ifne iflt ifge ifgt ifle if_icmpeq if_icmpne if_icmplt if_icmpge if_icmpgt if_icmple if_acmpeq if_acmpne goto jsr ret tableswitch lookupswitch ireturn lreturn freturn dreturn areturn return getstatic putstatic getfield putfield invokevirtual invokespecial invokestatic invokeinterface invokedynamic new newarray anewarray arraylength athrow checkcast instanceof monitorenter monitorexit wide multianewarray ifnull ifnonnull goto_w jsr_w ";
        int index;
        for (int n = 0; substring.length() > 0; substring = substring.substring(index + 1), ++n) {
            index = substring.indexOf(32);
            Instruction.BC_NAME[n] = substring.substring(0, index);
        }
        Instruction.BW = 4;
    }
    
    public abstract static class Switch extends Instruction
    {
        protected int apc;
        
        public abstract int getCaseCount();
        
        public abstract int getCaseValue(int p0);
        
        public abstract int getCaseLabel(int p0);
        
        public abstract void setCaseCount(int p0);
        
        public abstract void setCaseValue(int p0, int p1);
        
        public abstract void setCaseLabel(int p0, int p1);
        
        protected abstract int getLength(int p0);
        
        public int getDefaultLabel() {
            return this.intAt(0) + this.pc;
        }
        
        public void setDefaultLabel(int n) {
            this.setIntAt(0, n - this.pc);
        }
        
        protected int intAt(int n) {
            return Instruction.getInt(this.bytes, this.apc + n * 4);
        }
        
        protected void setIntAt(int n, int n2) {
            Instruction.setInt(this.bytes, this.apc + n * 4, n2);
        }
        
        protected Switch(byte[] array, int n, int n2) {
            super(array, n, n2, 0, 0);
            this.apc = alignPC(n + 1);
            this.special = true;
            this.length = this.getLength(this.getCaseCount());
        }
        
        public int getAlignedPC() {
            return this.apc;
        }
        
        @Override
        public String toString() {
            String str = super.toString() + " Default:" + Instruction.labstr(this.getDefaultLabel());
            for (int caseCount = this.getCaseCount(), i = 0; i < caseCount; ++i) {
                str = str + "\n\tCase " + this.getCaseValue(i) + ":" + Instruction.labstr(this.getCaseLabel(i));
            }
            return str;
        }
        
        public static int alignPC(int n) {
            while (n % 4 != 0) {
                ++n;
            }
            return n;
        }
    }
    
    public static class TableSwitch extends Instruction.Switch
    {
        public int getLowCase() {
            return this.intAt(1);
        }
        
        public int getHighCase() {
            return this.intAt(2);
        }
        
        @Override
        public int getCaseCount() {
            return this.intAt(2) - this.intAt(1) + 1;
        }
        
        @Override
        public int getCaseValue(int n) {
            return this.getLowCase() + n;
        }
        
        @Override
        public int getCaseLabel(int n) {
            return this.intAt(3 + n) + this.pc;
        }
        
        public void setLowCase(int n) {
            this.setIntAt(1, n);
        }
        
        public void setHighCase(int n) {
            this.setIntAt(2, n);
        }
        
        @Override
        public void setCaseLabel(int n, int n2) {
            this.setIntAt(3 + n, n2 - this.pc);
        }
        
        @Override
        public void setCaseCount(int n) {
            this.setHighCase(this.getLowCase() + n - 1);
            this.length = this.getLength(n);
        }
        
        @Override
        public void setCaseValue(int n, int lowCase) {
            if (n != 0) {
                throw new UnsupportedOperationException();
            }
            int caseCount = this.getCaseCount();
            this.setLowCase(lowCase);
            this.setCaseCount(caseCount);
            return;
        }
        
        TableSwitch(byte[] array, int n) {
            super(array, n, 170);
        }
        
        @Override
        protected int getLength(int n) {
            return this.apc - this.pc + (3 + n) * 4;
        }
    }
    
    public static class LookupSwitch extends Instruction.Switch
    {
        @Override
        public int getCaseCount() {
            return this.intAt(1);
        }
        
        @Override
        public int getCaseValue(int n) {
            return this.intAt(2 + n * 2 + 0);
        }
        
        @Override
        public int getCaseLabel(int n) {
            return this.intAt(2 + n * 2 + 1) + this.pc;
        }
        
        @Override
        public void setCaseCount(int n) {
            this.setIntAt(1, n);
            this.length = this.getLength(n);
        }
        
        @Override
        public void setCaseValue(int n, int n2) {
            this.setIntAt(2 + n * 2 + 0, n2);
        }
        
        @Override
        public void setCaseLabel(int n, int n2) {
            this.setIntAt(2 + n * 2 + 1, n2 - this.pc);
        }
        
        LookupSwitch(byte[] array, int n) {
            super(array, n, 171);
        }
        
        @Override
        protected int getLength(int n) {
            return this.apc - this.pc + (2 + n * 2) * 4;
        }
    }
    
    static class FormatException extends IOException
    {
        private static final long serialVersionUID = 3175572275651367015L;
        
        FormatException(String message) {
            super(message);
        }
    }
}
